FieldElement

Struct FieldElement 

pub struct FieldElement(pub(super) MontyFieldElement<FieldParams, { FieldParams::LIMBS }>);
Element in the finite field modulo p = 2^{384} − 2^{128} − 2^{96} + 2^{32} − 1.

Trait impls

Much of the important functionality is provided by traits from the [ff] crate:

  • [Field] represents elements of finite fields and provides:
    • [Field::random] generate a random field element
    • double, square, and invert operations
    • Bounds for [Add], [Sub], [Mul], and [Neg] (and *Assign equivalents)
    • Bounds for [ConditionallySelectable] from the subtle crate
  • PrimeField represents elements of prime fields and provides:
    • from_repr/to_repr for converting field elements from/to big integers.
    • MULTIPLICATIVE_GENERATOR and ROOT_OF_UNITY constants.

Please see the documentation for the relevant traits for more information.

Tuple Fields

0: MontyFieldElement<FieldParams, { FieldParams::LIMBS }>

Implementations

Source

impl FieldElement

pub const ZERO: Self

Zero element.

pub const ONE: Self

Multiplicative identity.

pub fn from_bytes( repr: &MontyFieldBytes<FieldParams, { FieldParams::LIMBS }>, ) -> CtOption<Self>

Create a FieldElement from a canonical byte representation using the field’s configured byte order.

pub fn from_slice(slice: &[u8]) -> Option<Self>

Decode FieldElement from a byte slice using the field’s configured byte order.

pub const fn from_hex_vartime(hex: &str) -> Self

Decode a FieldElement from hex-encoded bytes using the field’s configured byte order.

This is primarily intended for defining constants using hex literals.

Panics
  • When hex is malformed
  • When input is the wrong length
  • If input overflows the modulus
pub fn from_uint(uint: &U384) -> CtOption<Self>

Decode FieldElement from U384 converting it into Montgomery form:

w * R^2 * R^-1 mod p = wR mod p
pub const fn from_u64(w: u64) -> Self

Convert a u64 into a FieldElement.

pub fn to_bytes(self) -> MontyFieldBytes<FieldParams, { FieldParams::LIMBS }>

Returns the big-endian encoding of this FieldElement.

pub fn is_odd(&self) -> Choice

Determine if this FieldElement is odd in the SEC1 sense: self mod 2 == 1.

Returns

If odd, return Choice(1). Otherwise, return Choice(0).

pub fn is_even(&self) -> Choice

Determine if this FieldElement is even in the SEC1 sense: self mod 2 == 0.

Returns

If even, return Choice(1). Otherwise, return Choice(0).

pub fn is_zero(&self) -> Choice

Determine if this FieldElement is zero.

Returns

If zero, return Choice(1). Otherwise, return Choice(0).

pub const fn pow_vartime<const RHS_LIMBS: usize>( &self, exp: &Uint<RHS_LIMBS>, ) -> Self

Returns self^exp, where exp is a little-endian integer exponent.

This operation is variable time with respect to the exponent exp.

If the exponent is fixed, this operation is constant time.

pub const fn sqn_vartime(&self, n: usize) -> Self

Returns self^(2^n) mod p.

This operation is variable time with respect to the exponent n.

If the exponent is fixed, this operation is constant time.

Source

impl FieldElement

pub(crate) const fn from_uint_unchecked(w: U384) -> Self

Decode FieldElement from U384 converting it into Montgomery form.

Does not perform a check that the field element does not overflow the order.

Used incorrectly this can lead to invalid results!

pub const fn to_canonical(self) -> U384

Translate FieldElement out of the Montgomery domain, returning a U384 in canonical form.

pub const fn add(&self, rhs: &Self) -> Self

Add elements.

pub const fn double(&self) -> Self

Double element (add it to itself).

pub const fn sub(&self, rhs: &Self) -> Self

Subtract elements.

pub const fn multiply(&self, rhs: &Self) -> Self

Multiply elements.

pub const fn neg(&self) -> Self

Negate element.

pub const fn square(&self) -> Self

Compute modular square.

pub fn invert(&self) -> CtOption<Self>

Compute FieldElement inversion: 1 / self.

pub fn invert_vartime(&self) -> CtOption<Self>

Compute FieldElement inversion: 1 / self in variable-time.

pub const fn const_invert(&self) -> Self

Compute field inversion as a const fn. Panics if self is zero.

This is mainly intended for inverting constants at compile time.